@@ -48,6 +48,7 @@ d_PRIu64='undef'
 d_PRIx64='undef'
 d_SCNfldbl='undef'
 d__fwalk='undef'
+d_accept4='undef'
 d_access='undef'
 d_accessx='undef'
 d_acosh='undef'
@@ -123,6 +124,7 @@ d_double_style_vax='undef'
 d_drand48_r='undef'
 d_drand48proto='undef'
 d_dup2='undef'
+d_dup3='undef'
 d_eaccess='undef'
 d_endgrent='undef'
 d_endgrent_r='undef'
@@ -328,6 +330,7 @@ d_memrchr='undef'
 d_mkdir='define'
 d_mkdtemp='undef'
 d_mkfifo='undef'
+d_mkostemp='undef'
 d_mkstemp='undef'
 d_mkstemps='undef'
 d_mktime64='undef'
@@ -372,6 +375,7 @@ d_pathconf='undef'
 d_pause='undef'
 d_perl_otherlibdirs='undef'
 d_phostname='undef'
+d_pipe2='undef'
 d_pipe='undef'
 d_poll='undef'
 d_portable='undef'
